/*
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.application;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.w3c.dom.Document;

import de.todesbaum.freemule.io.StreamCopier;
import de.todesbaum.freemule.util.Hex;
import de.todesbaum.freemule.util.SimpleXML;
import de.todesbaum.freemule.util.XML;
import de.todesbaum.util.freenet.fcp.FcpException;
import de.todesbaum.util.freenet.fcp.FcpRuntimeException;


/**
 * Service responsible for downloading files.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:DownloadService.java 220 2006-03-06 14:46:23Z bombe $
 */
public class DownloadService extends AbstractEngineService {

	public static final String NAME = "Download Service";

	/**
	 * The file suffix for download properties files.
	 */
	private static final String PROPERTIES_SUFFIX = ".properties";

	/**
	 * The file suffix for download data files.
	 */
	private static final String DATA_SUFFIX = ".data";

	/**
	 * A {@link List} containing all registered {@link DownloadListener}s.
	 */
	private List<DownloadListener> downloadListeners;

	/**
	 * The temporary directory to store partial downloads and check blocks in.
	 */
	private String tempDirectory;

	/**
	 * The incoming directory to store completely downloaded files in.
	 */
	private String incomingDirectory;

	/**
	 * The current {@link List} of {@link FreenetFile}s.
	 */
	private volatile List<FreenetFile> downloads;

	/**
	 * A random number generator, used for deciding what blocks to download.
	 */
	private Random random;

	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return NAME;
	}

	/**
	 * Adds a {@link DownloadListener} to the list of registered listeners.
	 * 
	 * @param downloadListener
	 *            The {@link DownloadListener} to add
	 */
	public void addDownloadListener(DownloadListener downloadListener) {
		if (!downloadListeners.contains(downloadListener)) {
			downloadListeners.add(downloadListener);
		}
	}

	/**
	 * Removes a {@link DownloadListener} from the list of registered listeners.
	 * 
	 * @param downloadListener
	 *            The {@link DownloadListener} to remove
	 */
	public void removeDownloadListener(DownloadListener downloadListener) {
		downloadListeners.remove(downloadListener);
	}

	/**
	 * Notifies listeners that a new download has been added.
	 * 
	 * @param download
	 *            The download that was added
	 */
	private void fireDownloadAdded(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadAdded(download);
		}
	}

	/**
	 * Notifies listeners that a download was started.
	 * 
	 * @param download
	 *            The download that was started
	 */
	public void fireDownloadStarted(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadStarted(download);
		}
	}

	/**
	 * Notifies listeners that a download was stopped.
	 * 
	 * @param download
	 *            The download that was stopped
	 */
	public void fireDownloadStopped(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadStopped(download);
		}
	}

	/**
	 * Notifies listeners that the given download was removed from the download list.
	 * 
	 * @param download
	 *            The download to remove
	 */
	private void fireDownloadRemoved(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadRemoved(download);
		}
	}

	/**
	 * Notifies listeners that a download is complete, i.e. moved to the incoming directory.
	 * 
	 * @param download
	 *            The download that is complete.
	 */
	private void fireDownloadComplete(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadComplete(download);
		}
	}

	/**
	 * Notifies listeners that the renaming operation from temporary to incoming directory failed when trying the hard way, i.e. simply copying it.
	 * 
	 * @param download
	 *            The download that failed to complete
	 * @param exception
	 *            The exception that caused completion to fail
	 */
	private void fireDownloadCompletionFailed(FreenetFile download, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadCompletionFailed(download, exception);
		}
	}

	/**
	 * Notifies listeners that a download is about to complete, i.e. being moved to the incoming directory.
	 * 
	 * @param download
	 *            The download that is completing
	 */
	private void fireDownloadCompleting(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.downloadCompleting(download);
		}
	}

	/**
	 * Notifies listeners that the loading of unfinished downloads from <code>tempDir</code> is starting.
	 * 
	 * @param tempDir
	 *            The temporary directory to load downloads from
	 */
	private void fireLoadDownloadsStarted(String tempDir) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.loadDownloadsStarted(tempDir);
		}
	}

	/**
	 * Notifies listeners that downloads could not be loaded from <code>tempDir</code>.
	 * 
	 * @param tempDir
	 *            The temporary directory to load downloads from
	 * @param exception
	 *            The exception that prevented loading
	 */
	private void fireLoadDownloadsFailed(String tempDir, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.loadDownloadsFailed(tempDir, exception);
		}
	}

	/**
	 * Notifies listeners that all downloads from <code>tempDir</code> have been loaded.
	 * 
	 * @param tempDir
	 *            The temporary directory downloads were loaded from
	 */
	private void fireLoadDownloadsComplete(String tempDir) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.loadDownloadsComplete(tempDir);
		}
	}

	/**
	 * Notifies listeners that loading a download from the properties file has started.
	 * 
	 * @param filename
	 *            The name of the properties file
	 */
	private void fireLoadDownloadStarted(String filename) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.loadDownloadStarted(filename);
		}
	}

	/**
	 * Notifies listeners that loading a download from the properties file has failed.
	 * 
	 * @param filename
	 *            The name of the properties file
	 * @param exception
	 *            The exception that caused loading to fail
	 */
	private void fireLoadDownloadFailed(String filename, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.loadDownloadFailed(filename, exception);
		}
	}

	/**
	 * Notifies listeners that loading a download from the properties file has completed successfully.
	 * 
	 * @param filename
	 *            The name of the properties file
	 * @param download
	 *            The download that was loaded
	 */
	private void fireLoadDownloadComplete(String filename, FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.loadDownloadComplete(filename, download);
		}
	}

	/**
	 * Notifies listeners that saving a downloaded block of data has started.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The number of the block
	 */
	private void fireSaveBlockStarted(FreenetFile download, int blockNumber) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveBlockStarted(download, blockNumber);
		}
	}

	/**
	 * Notifies listeners that saving a downloaded block of dta has failed.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The number of the block
	 */
	private void fireSaveBlockFailed(FreenetFile download, int blockNumber, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveBlockFailed(download, blockNumber, exception);
		}
	}

	/**
	 * Notifies listeners that saving a downloaded block of data has completed successfully.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The number of the block
	 */
	private void fireSaveBlockComplete(FreenetFile download, int blockNumber) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveBlockComplete(download, blockNumber);
		}
	}

	/**
	 * Notifies listeners that saving the downloads to <code>tempDir</code> is about to start.
	 * 
	 * @param tempDir
	 *            The temporary directory to save downloads to
	 */
	private void fireSaveDownloadsStarted(String tempDir) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveDownloadsStarted(tempDir);
		}
	}

	/**
	 * Notifies listeners that saving the downloads to <code>tempDir</code> failed.
	 * 
	 * @param tempDir
	 *            The temporary directory to save the downloads to
	 * @param exception
	 *            The exception that prevented saving
	 */
	private void fireSaveDownloadsFailed(String tempDir, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveDownloadsFailed(tempDir, exception);
		}
	}

	/**
	 * Notifies listeners that all downloads have been saved to <code>tempDir</code>.
	 * 
	 * @param tempDir
	 *            The temporary directory the downloads have been saved to
	 */
	private void fireSaveDownloadsComplete(String tempDir) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveDownloadsComplete(tempDir);
		}
	}

	/**
	 * Notifies listeners that saving a download has started.
	 * 
	 * @param download
	 *            The download to save
	 */
	private void fireSaveDownloadStarted(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveDownloadStarted(download);
		}
	}

	/**
	 * Notifies listeners that saving a download has failed.
	 * 
	 * @param download
	 *            The download to save
	 * @param exception
	 *            The exception that caused saving to fail
	 */
	private void fireSaveDownloadFailed(FreenetFile download, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveDownloadFailed(download, exception);
		}
	}

	/**
	 * Notifies listeners that saving a download completed successfully.
	 * 
	 * @param download
	 *            The download to save
	 */
	private void fireSaveDownloadComplete(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.saveDownloadComplete(download);
		}
	}

	/**
	 * Notifies listeners that the manifest for the specified download was requested from the {@link EngineService}.
	 * 
	 * @see DownloadListener#manifestDownloadStarted(FreenetFile)
	 * @param download
	 *            The download the manifest was requested from
	 */
	private void fireManifestDownloadRequested(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.manifestDownloadStarted(download);
		}
	}

	/**
	 * Notifies listeners that the manifest for the specified download was completely received.
	 * 
	 * @see DownloadListener#manifestDownloadComplete(FreenetFile)
	 * @param download
	 *            The download the manifest was requested from
	 */
	private void fireManifestDownloadComplete(FreenetFile download) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.manifestDownloadComplete(download);
		}
	}

	/**
	 * Notifies listeners that the download of manifest for the specified download failed.
	 * 
	 * @see DownloadListener#manifestDownloadFailed(FreenetFile, Exception)
	 * @param download
	 *            The download the manifest was requested from
	 * @param exception
	 *            The exception that prevented download
	 */
	private void fireManifestDownloadFailed(FreenetFile download, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.manifestDownloadFailed(download, exception);
		}
	}

	/**
	 * Notifies listeners that the download of a block of data is about to start.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The block number of the block
	 */
	private void fireBlockDownloadStarted(FreenetFile download, int blockNumber) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.blockDownloadStarted(download, blockNumber);
		}
	}

	/**
	 * Notifies listeners that the download of a block of data failed.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The block number of the block
	 * @param exception
	 *            The exception that prevented downloading
	 */
	private void fireBlockDownloadFailed(FreenetFile download, int blockNumber, Exception exception) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.blockDownloadFailed(download, blockNumber, exception);
		}
	}

	/**
	 * Notifies listeners that the download of a block of data is complete.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The block number of the block
	 */
	private void fireBlockDownloadComplete(FreenetFile download, int blockNumber) {
		for (DownloadListener downloadListener: downloadListeners) {
			downloadListener.blockDownloadComplete(download, blockNumber);
		}
	}

	/**
	 * Sets the temporary directory for the download service. The temporary directory is used to store download properties and check blocks.
	 * 
	 * @param tempDirectory
	 *            The temporary directory to use
	 */
	public void setTempDirectory(String tempDirectory) {
		this.tempDirectory = tempDirectory;
	}

	/**
	 * Sets the incoming directory for the download service. The incoming directory is used to move complete files into.
	 * 
	 * @param incomingDirectory
	 *            The incoming directory to use
	 */
	public void setIncomingDirectory(String incomingDirectory) {
		this.incomingDirectory = incomingDirectory;
	}

	/**
	 * Adds a new download to the list of downloads.
	 * 
	 * @param key
	 *            The key of the download
	 * @param name
	 *            The name of the download
	 */
	public synchronized void addDownload(String key, String name) {
		addDownload(key, name, 0);
	}

	/**
	 * Adds a new download to the list of downloads.
	 * 
	 * @param key
	 *            The key of the download
	 * @param name
	 *            The name of the download
	 * @param length
	 *            The length of the download, if known
	 */
	public synchronized boolean addDownload(String key, String name, long length) {
		for (FreenetFile download: downloads) {
			if (key.equals(download.getKey())) {
				return false;
			}
		}
		FreenetFile download = new FreenetFile();
		download.setKey(key);
		download.setName(name);
		download.setStatus(FileStatus.activated);
		download.setManifestStatus(BlockStatus.notAvailable);
		download.setLength(length);
		// search new basename
		int index = 0;
		while (new File(tempDirectory, Hex.toHex(index) + PROPERTIES_SUFFIX).exists()) {
			index++;
		}
		download.setDataFilename(new File(tempDirectory, Hex.toHex(index) + DATA_SUFFIX).getAbsolutePath());
		download.setPropertiesFilename(new File(tempDirectory, Hex.toHex(index) + PROPERTIES_SUFFIX).getAbsolutePath());
		downloads.add(download);
		saveTempDirectory();
		fireDownloadAdded(download);
		synchronized (syncObject) {
			syncObject.notify();
		}
		return true;
	}

	/**
	 * Removes a download from the list of downloads.
	 * 
	 * @param download
	 *            The download to remove
	 */
	public synchronized void removeDownload(FreenetFile download) {
		if (downloads.remove(download)) {
			File dataFile = new File(download.getDataFilename());
			dataFile.delete();
			File propertiesFile = new File(download.getPropertiesFilename());
			propertiesFile.delete();
			saveTempDirectory();
			fireDownloadRemoved(download);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void init() {
		super.init();
		downloadListeners = new ArrayList<DownloadListener>();
		downloads = new ArrayList<FreenetFile>();
		random = new Random(System.currentTimeMillis());
		tempDirectory = System.getProperty("user.home") + "/.freeMule/temp";
		incomingDirectory = System.getProperty("user.home") + "/.freeMule/incoming";
		createTempAndIncomingDirectories();
		loadTempDirectory();
		setWaitPeriod(1000);
	}

	/**
	 * {@inheritDoc}
	 */
	public void start() {
		super.start();
	}

	/**
	 * {@inheritDoc}
	 */
	public void stop() {
		saveTempDirectory();
		super.stop();
	}

	/**
	 * {@inheritDoc}
	 */
	public void destroy() {
		downloads = null;
		incomingDirectory = null;
		tempDirectory = null;
		super.destroy();
	}

	/**
	 * Creates the temporary and incoming directories.
	 */
	private void createTempAndIncomingDirectories() {
		File directory = new File(tempDirectory);
		if (!directory.exists()) {
			directory.mkdirs();
		} else {
			if (!directory.isDirectory()) {
				// TODO - signal error
			}
		}
		directory = new File(incomingDirectory);
		if (!directory.exists()) {
			directory.mkdirs();
		} else {
			if (!directory.isDirectory()) {
				// TODO - signal error
			}
		}
	}

	/**
	 * Kicks off the next download. Also, checks for segments that can be decoded and complete files that can be moved to the incoming directory.
	 */
	protected void work() {
		List<FreenetFile> downloadToRemove = new ArrayList<FreenetFile>();
		synchronized (this) {
			for (FreenetFile freenetFile: downloads) {
				setStatus("Checking for complete files");
				if (checkForFileCompletion(freenetFile)) {
					downloadToRemove.add(freenetFile);
				}
			}
			for (FreenetFile freenetFile: downloadToRemove) {
				removeDownload(freenetFile);
			}
		}
		DownloadInformation downloadInformation = null;
		synchronized (this) {
			downloadInformation = findNextDownload();
			if (downloadInformation != null) {
				if (downloadInformation.isManifest()) {
					downloadInformation.getDownload().setManifestStatus(BlockStatus.requesting);
				} else {
					downloadInformation.getDownload().setBlockStatus(downloadInformation.getBlockNumber(), BlockStatus.requesting);
				}
			}
		}
		if (downloadInformation != null) {
			FreenetFile freenetFile = downloadInformation.getDownload();
			String key = null;
			int blockNumber = -1;
			if (downloadInformation.isManifest()) {
				key = freenetFile.getKey();
			} else {
				blockNumber = downloadInformation.getBlockNumber();
				key = freenetFile.getBlockKey(blockNumber);
			}

			if (downloadInformation.isManifest()) {
				try {
					setStatus("Downloading Manifest for " + freenetFile.getName());
					fireManifestDownloadRequested(freenetFile);
					Data blockData = engine.getKey(key, hopsToLive);
					if (blockData.getData().length == 0) {
						decodeBlockList(freenetFile, blockData.getMetadata());
						saveTempDirectory();
					} else {
						saveFile(freenetFile, blockData.getData());
						fireDownloadComplete(freenetFile);
						saveTempDirectory();
					}
					freenetFile.setManifestStatus(BlockStatus.available);
					fireManifestDownloadComplete(freenetFile);
				} catch (FcpRuntimeException fre1) {
					freenetFile.setManifestStatus(BlockStatus.notAvailable);
					fireManifestDownloadFailed(freenetFile, fre1);
				} catch (IOException ioe1) {
					freenetFile.setManifestStatus(BlockStatus.notAvailable);
					fireManifestDownloadFailed(freenetFile, ioe1);
				} catch (FcpException fe1) {
					fireServiceAborted(fe1);
					stopping = true;
				} catch (EngineStoppedException ese1) {
					fireServiceAborted(ese1);
					stopping = true;
				}
			} else {
				setStatus("Download Block for " + freenetFile.getName() + ", Block " + blockNumber);
				fireBlockDownloadStarted(freenetFile, blockNumber);
				try {
					Data blockData = engine.getKey(key, hopsToLive);
					freenetFile.setBlockStatus(blockNumber, BlockStatus.saving);
					if (saveBlock(freenetFile, blockNumber, blockData.getData())) {
						freenetFile.setBlockStatus(blockNumber, BlockStatus.available);
					} else {
						freenetFile.setBlockStatus(blockNumber, BlockStatus.notAvailable);
					}
					fireBlockDownloadComplete(freenetFile, blockNumber);
				} catch (FcpRuntimeException fre1) {
					freenetFile.setBlockStatus(blockNumber, BlockStatus.notAvailable);
					fireBlockDownloadFailed(freenetFile, blockNumber, fre1);
				} catch (IOException ioe1) {
					freenetFile.setBlockStatus(blockNumber, BlockStatus.notAvailable);
					fireBlockDownloadFailed(freenetFile, blockNumber, ioe1);
				} catch (FcpException fe1) {
					fireServiceAborted(fe1);
					stopping = true;
				} catch (EngineStoppedException ese1) {
					fireServiceAborted(ese1);
					stopping = true;
				}
			}
		}
	}

	/**
	 * Saves the specified data to the file given in <code>download</code> and finish it.
	 * 
	 * @param download
	 *            The download to complete
	 * @param data
	 *            The data of the file
	 */
	private void saveFile(FreenetFile download, byte[] data) {
		fireDownloadCompleting(download);
		download.setLength(data.length);
		try {
			File tempFile = new File(tempDirectory, download.getDataFilename());
			FileOutputStream fileOutput = new FileOutputStream(tempFile);
			fileOutput.write(data);
			fileOutput.close();
			moveFileToIncoming(download);
		} catch (IOException ioe1) {
			fireDownloadCompletionFailed(download, ioe1);
		}
	}

	/**
	 * Checks whether a download is complete (i.e. all segments have been decoded). If a download is complete, it's moved to the incoming directory
	 * and appropriate notifications are sent.
	 * 
	 * @see DownloadListener#downloadCompleting(FreenetFile)
	 * @param download
	 *            The download to check for completion
	 */
	private boolean checkForFileCompletion(FreenetFile download) {
		if (download.getStatus() != FileStatus.complete) {
			boolean hasAllBlocks = true;
			for (int blockIndex = 0, blockCount = download.getBlockCount(); hasAllBlocks && (blockIndex < blockCount); blockIndex++) {
				hasAllBlocks &= download.getBlockStatus(blockIndex) == BlockStatus.available;
			}
			if (hasAllBlocks) {
				download.setStatus(FileStatus.complete);
				fireDownloadCompleting(download);
				if (!moveFileToIncoming(download)) {
					download.setStatus(FileStatus.error);
				}
				fireDownloadComplete(download);
				return true;
			}
		}
		return false;
	}

	/**
	 * Moves the temporary file for <code>download</code> to the incoming directory.
	 * 
	 * @see DownloadListener#downloadCompletionFailed(FreenetFile, java.lang.Exception)
	 * @see DownloadListener#downloadComplete(FreenetFile)
	 * @param download
	 *            The download to move
	 */
	private boolean moveFileToIncoming(FreenetFile download) {
		File tempFile = new File(download.getDataFilename());
		String filename = download.getName();
		int lastSlash = filename.lastIndexOf('/');
		if (lastSlash > -1) {
			filename = filename.substring(lastSlash);
		}
		File newFile = new File(incomingDirectory, filename);
		int index = 1;
		while (newFile.exists()) {
			int lastDot = filename.lastIndexOf('.');
			String newFilename = filename.substring(0, lastDot) + " (" + index + ")" + filename.substring(lastDot);
			newFile = new File(incomingDirectory, newFilename);
			index++;
		}
		if (!tempFile.renameTo(newFile)) {
			try {
				FileOutputStream fileOutput = new FileOutputStream(newFile);
				FileInputStream fileInput = new FileInputStream(tempFile);
				long remaining = download.getLength();
				StreamCopier.copy(fileInput, fileOutput, remaining);
				fileInput.close();
				fileOutput.close();
				if (newFile.length() != download.getLength()) {
					download.setStatus(FileStatus.error);
					fireDownloadCompletionFailed(download, new IOException("File sizes differ."));
					return false;
				}
				download.setStatus(FileStatus.complete);
				tempFile.delete();
				File propertiesFile = new File(tempDirectory, download.getPropertiesFilename());
				propertiesFile.delete();
				return true;
			} catch (IOException ioe1) {
				download.setStatus(FileStatus.error);
				fireDownloadCompletionFailed(download, ioe1);
			}
		} else {
			download.setStatus(FileStatus.complete);
			File propertiesFile = new File(tempDirectory, download.getPropertiesFilename());
			propertiesFile.delete();
			return true;
		}
		return false;
	}

	/**
	 * Scans the temporary directory for download properties files and loads the downloads.
	 */
	private void loadTempDirectory() {
		fireLoadDownloadsStarted(tempDirectory);
		// first, scan temp dir
		File tempDir = new File(tempDirectory);
		if (!tempDir.exists() || !tempDir.isDirectory()) {
			fireLoadDownloadsFailed(tempDirectory, new IOException("\"" + tempDirectory + "\" does not denote an existing directory"));
			return;
		}
		File[] files = tempDir.listFiles(new FileFilter() {

			public boolean accept(File pathname) {
				if (pathname.isDirectory())
					return false;
				return pathname.getName().endsWith(PROPERTIES_SUFFIX);
			}
		});
		if (files == null) {
			fireLoadDownloadsFailed(tempDirectory, new IOException("\"" + tempDirectory + "\" can not be read"));
			return;
		}
		for (File propertiesFile: files) {
			fireLoadDownloadStarted(propertiesFile.getAbsolutePath());
			if (propertiesFile.length() > Integer.MAX_VALUE) {
				continue;
			}
			ByteArrayOutputStream xmlOutput = new ByteArrayOutputStream((int) propertiesFile.length());
			try {
				StreamCopier.copy(new FileInputStream(propertiesFile), xmlOutput, propertiesFile.length());
				xmlOutput.close();
				byte[] xmlBuffer = xmlOutput.toByteArray();
				SimpleXML downloadXML = SimpleXML.fromDocument(XML.transformToDocument(xmlBuffer));
				FreenetFile freenetFile = FreenetFile.createFromXML(downloadXML);
				if (freenetFile.getManifestStatus() != BlockStatus.available) {
					freenetFile.setManifestStatus(BlockStatus.notAvailable);
				}
				if (freenetFile.getManifestStatus() == BlockStatus.available) {
					for (int blockIndex = 0, blockCount = freenetFile.getBlockCount(); blockIndex < blockCount; blockIndex++) {
						if (freenetFile.getBlockStatus(blockIndex) != BlockStatus.available) {
							freenetFile.setBlockStatus(blockIndex, BlockStatus.notAvailable);
						}
					}
				}
				downloads.add(freenetFile);
				fireLoadDownloadComplete(propertiesFile.getAbsolutePath(), freenetFile);
			} catch (IOException ioe1) {
				fireLoadDownloadFailed(propertiesFile.getAbsolutePath(), ioe1);
				continue;
			}
		}
		fireLoadDownloadsComplete(tempDirectory);
	}

	/**
	 * Saves a block that was downloaded to the appropriate file.
	 * 
	 * @param download
	 *            The download of the block
	 * @param blockNumber
	 *            The number of block
	 * @param blockData
	 *            The data to save
	 * @return <code>true</code> if the data could be saved, <code>false</code> otherwise
	 */
	private boolean saveBlock(FreenetFile download, int blockNumber, byte[] blockData) {
		fireSaveBlockStarted(download, blockNumber);
		OutputStream blockOutput = null;
		try {
			blockOutput = download.getBlockOutputStream(blockNumber);
			StreamCopier.copy(new ByteArrayInputStream(blockData), blockOutput, blockData.length);
			blockOutput.close();
			blockOutput = null;
			fireSaveBlockComplete(download, blockNumber);
			return true;
		} catch (IOException ioe1) {
			fireSaveBlockFailed(download, blockNumber, ioe1);
		} finally {
			if (blockOutput != null) {
				try {
					blockOutput.close();
				} catch (IOException ioe1) {
				}
				blockOutput = null;
			}
		}
		return false;
	}

	/**
	 * Saves the status information for all downloads to the {@link #tempDirectory temporary directory}.
	 * 
	 * @see DownloadListener#saveDownloadsStarted(String)
	 * @see DownloadListener#saveDownloadsFailed(String, Exception)
	 * @see DownloadListener#saveDownloadsComplete(String)
	 * @see DownloadListener#saveDownloadComplete(FreenetFile)
	 * @see DownloadListener#saveDownloadFailed(FreenetFile, Exception)
	 */
	public synchronized void saveTempDirectory() {
		for (FreenetFile download: downloads) {
			fireSaveDownloadStarted(download);
			String downloadPropertiesFilename = download.getPropertiesFilename();
			SimpleXML downloadXML = download.getXML("download");
			File downloadPropertiesFile = new File(downloadPropertiesFilename);
			try {
				FileOutputStream downloadPropertiesFileOutput = new FileOutputStream(downloadPropertiesFile);
				downloadPropertiesFileOutput.write(XML.transformToByteArray(downloadXML.getDocument()));
				downloadPropertiesFileOutput.close();
			} catch (IOException ioe1) {
				fireSaveDownloadFailed(download, ioe1);
			}
		}
		fireSaveDownloadsComplete(tempDirectory);
	}

	/**
	 * Finds the next available download. Should (in a multi-threaded) environment only be called when the download given by a previous call was
	 * already started (and the download's status information has already been updated).
	 * <p>
	 * 
	 * @return The information what to download next, or <code>null</code> if there is nothing to do
	 */
	private synchronized DownloadInformation findNextDownload() {
		List<FreenetFile> availableDownloads = findAvailableDownloads();
		if (availableDownloads.size() > 0) {
			int downloadNumber = random.nextInt(availableDownloads.size());
			FreenetFile nextDownload = downloads.get(downloadNumber);
			if (nextDownload.getManifestStatus() == BlockStatus.notAvailable) {
				return new DownloadInformation(nextDownload);
			}
			List<Integer> availableBlocks = findAvailableBlocks(nextDownload);
			int blockNumber = random.nextInt(availableBlocks.size());
			int block = availableBlocks.get(blockNumber);
			return new DownloadInformation(nextDownload, block);
		}
		return null;
	}

	/**
	 * Find downloads that do not yet have a manifest (resulting in segmentation information).
	 * 
	 * @return A {@link List} containing the downloads without manifest
	 */
	private List<FreenetFile> findAvailableDownloads() {
		List<FreenetFile> availableDownloads = new ArrayList<FreenetFile>();
		for (FreenetFile download: downloads) {
			if (download.getStatus() != FileStatus.activated) {
				continue;
			}
			if (download.getManifestStatus() == BlockStatus.notAvailable) {
				availableDownloads.add(download);
				continue;
			}
			int blockCount = download.getBlockCount();
			boolean missBlocks = false;
			for (int blockIndex = 0; !missBlocks && (blockIndex < blockCount); blockIndex++) {
				if (download.getBlockStatus(blockIndex) == BlockStatus.notAvailable) {
					missBlocks = true;
				}
			}
			if (missBlocks) {
				availableDownloads.add(download);
			}
		}
		return availableDownloads;
	}

	/**
	 * Scans a segment within a download for blocks that are still missing.
	 * 
	 * @param download
	 *            The download of the segment
	 * @return A {@link List} with the numbers of missing blocks
	 */
	private List<Integer> findAvailableBlocks(FreenetFile download) {
		List<Integer> availableBlocks = new ArrayList<Integer>();
		int blockCount = download.getBlockCount();
		for (int blockIndex = 0; blockIndex < blockCount; blockIndex++) {
			if (download.getBlockStatus(blockIndex) == BlockStatus.notAvailable) {
				availableBlocks.add(blockIndex);
			}
		}
		return availableBlocks;
	}

	/**
	 * Returns the current download list.
	 * 
	 * @return The current download list
	 */
	public FreenetFile[] getDownloads() {
		return downloads.toArray(new FreenetFile[downloads.size()]);
	}

	/**
	 * Returns the index of a specific download in the list.
	 * 
	 * @param download
	 *            The download to look for
	 * @return The index of the download, or <code>-1</code> if the download could not be found
	 */
	public int getDownloadIndex(FreenetFile download) {
		return downloads.indexOf(download);
	}

	private void decodeBlockList(FreenetFile freenetFile, byte[] metadata) {
		Document fileDocument = XML.transformToDocument(metadata);
		SimpleXML fileXML = SimpleXML.fromDocument(fileDocument);
		FreenetFile storedFile = FreenetFile.createFromXML(fileXML);
		freenetFile.setLength(storedFile.getLength());
		for (int blockIndex = 0, blockCount = storedFile.getBlockCount(); blockIndex < blockCount; blockIndex++) {
			freenetFile.setBlockKey(blockIndex, storedFile.getBlockKey(blockIndex));
		}
	}

	public void stopDownload(FreenetFile download) {
		int index = getDownloadIndex(download);
		if (index != -1) {
			download.setStatus(FileStatus.deactivated);
			fireDownloadStopped(download);
		}
	}

	public void resumeDownload(FreenetFile download) {
		int index = getDownloadIndex(download);
		if (index != -1) {
			download.setStatus(FileStatus.activated);
			fireDownloadStarted(download);
		}
	}

}
