/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.application;

import de.todesbaum.freemule.util.Hex;
import de.todesbaum.freemule.util.SimpleXML;

/**
 * A shared file is an extension to {@link FreenetFile} that holds local
 * information, such as timestamp, the directory, a comment, a rating, and the
 * number of requests and uploads for this file.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden </a>
 * @version $Id:SharedFile.java 220 2006-03-06 14:46:23Z bombe $
 */
public class SharedFile extends FreenetFile {

	/**
	 * Text constants for the {@link #rating} value.
	 */
	public static final String[] RATING = new String[] { "Not rated", "Corrupt / Fake", "Bad Quality", "Okay", "Excellent" };

	/**
	 * The timestamp for this file.
	 */
	private long timestamp;

	/**
	 * The name of the shared directory that contains this file.
	 */
	private String directory;

	/**
	 * The user-given comment for this file.
	 */
	private String comment;

	/**
	 * The user-given rating of this file.
	 */
	private int rating;

	/**
	 * The number of requests.
	 */
	private int requests;

	/**
	 * The number of uploads.
	 */
	private int uploads;

	/**
	 * Empty constructor for bean-like handling.
	 * 
	 */
	public SharedFile() {
	}

	/**
	 * Copy-constructor for an existing {@link SharedFile}.
	 * 
	 * @see FreenetFile#FreenetFile(FreenetFile)
	 * @param freenetFile
	 *            The file to copy
	 */
	public SharedFile(FreenetFile freenetFile) {
		super(freenetFile);
		if (freenetFile instanceof SharedFile) {
			SharedFile sharedFile = (SharedFile) freenetFile;
			this.timestamp = sharedFile.timestamp;
			this.directory = sharedFile.directory;
			this.comment = sharedFile.comment;
			this.rating = sharedFile.rating;
			this.requests = sharedFile.requests;
			this.uploads = sharedFile.uploads;
		}
	}

	/**
	 * Returns the name of the shared directory that holds this file.
	 * <p>
	 * <string>Note:</strong> This is not necessarily the directory that
	 * contains this file as {@link #getName()} may return additional path
	 * information!
	 * 
	 * @return The name of the shared directory
	 */
	public String getDirectory() {
		return directory;
	}

	/**
	 * Sets the name of the shared directory that holds this file.
	 * 
	 * @param directory
	 *            The name of the shared directory
	 */
	public void setDirectory(String directory) {
		this.directory = directory;
	}

	/**
	 * Returns the timestamp of this file, as in
	 * {@link java.io.File#lastModified()}.
	 * 
	 * @return The timestamp of this file
	 */
	public long getTimestamp() {
		return timestamp;
	}

	/**
	 * Sets the timestamp of this file.
	 * 
	 * @see java.io.File#lastModified()
	 * @param timestamp
	 *            The timestamp to set
	 */
	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}

	/**
	 * Returns the number of requests for this file.
	 * 
	 * @return The number of requests
	 */
	public int getRequests() {
		return requests;
	}

	/**
	 * Sets the number of requests for this file.
	 * 
	 * @param requests
	 *            The number of requests
	 */
	public void setRequests(int requests) {
		this.requests = requests;
	}

	/**
	 * Returns the number of uploads for this file.
	 * 
	 * @return The number of uploads
	 */
	public int getUploads() {
		return uploads;
	}

	/**
	 * Sets the number of uploads for this file.
	 * 
	 * @param uploads
	 *            The number of uploads
	 */
	public void setUploads(int uploads) {
		this.uploads = uploads;
	}

	/**
	 * Returns the comment of this file.
	 * 
	 * @return The comment of this file
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * Returns the rating of this file.
	 * 
	 * @return The rating of this file
	 */
	public int getRating() {
		return rating;
	}

	/**
	 * Sets the comment of this file.
	 * 
	 * @param comment
	 *            The comment of this file
	 */
	public void setComment(String comment) {
		this.comment = (comment != null) ? comment : "";
	}

	/**
	 * Sets the rating of this file.
	 * 
	 * @param rating
	 *            The rating of this file
	 */
	public void setRating(int rating) {
		this.rating = rating;
	}
	
	@Override
	public SimpleXML getXML(String elementName) {
		SimpleXML resultXML = super.getXML(elementName);
		resultXML.append("timestamp", Hex.toHex(timestamp));
		resultXML.append("directory", directory);
		resultXML.append("comment", comment);
		resultXML.append("requests", Hex.toHex(requests));
		resultXML.append("uploads", Hex.toHex(uploads));
		resultXML.append("rating", Hex.toHex(rating));
		return resultXML;
	}

	public static SharedFile createFromXML(SimpleXML sharedFileXML) {
		SharedFile sharedFile = new SharedFile(FreenetFile.createFromXML(sharedFileXML));
		sharedFile.setTimestamp(Hex.toLong(sharedFileXML.getNode("timestamp").getValue()));
		sharedFile.setDirectory(sharedFileXML.getNode("directory").getValue());
		sharedFile.setComment(sharedFileXML.getNode("comment").getValue());
		sharedFile.setRequests((int) Hex.toLong(sharedFileXML.getNode("requests").getValue()));
		sharedFile.setUploads((int) Hex.toLong(sharedFileXML.getNode("uploads").getValue()));
		sharedFile.setRating((int) Hex.toLong(sharedFileXML.getNode("rating").getValue()));
		return sharedFile;
	}
	
}
