/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.application;

import java.util.EventListener;

/**
 * Interface for listeners that want to be notified on shared file events. Those
 * events include loading of the list of shared files, scanning of the shared
 * directories, hashing new and modified files.
 * 
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id:SharedFileListener.java 220 2006-03-06 14:46:23Z bombe $
 */
public interface SharedFileListener extends EventListener {

	/**
	 * Notifies listeners that the loading of shared files has started.
	 * 
	 * @param filename
	 *            The filename from which the shared files are loaded
	 */
	public void sharedFileLoadingStarted(String filename);

	/**
	 * Notifies listeners that the loading of shared files has failed.
	 * 
	 * @param filename
	 *            The filename from which the shared files are loaded
	 * @param exception
	 *            The exception that caused loading to fail
	 */
	public void sharedFileLoadingFailed(String filename, Exception exception);

	/**
	 * Notifies listeners that the loading of shared files has completed.
	 * 
	 * @param filename
	 *            The filename from which the shared files are loaded
	 */
	public void sharedFileLoadingComplete(String filename);

	/**
	 * Notifies listeners that a file is no longer shared. This happens if you
	 * shared a file last time freeMule ran and afterwards either changed the
	 * shared directories or deleted/moved the file.
	 * 
	 * @param filename
	 *            The name of the file that is no longer shared
	 */
	public void sharedFileUnsharedFile(String filename);

	/**
	 * Notifies listeners that a directory is no longer shared. This happens if
	 * you shared a directory last time freeMule ran and afterwards changed the
	 * shared directories.
	 * 
	 * @param directory
	 *            The name of the directory that is no longer shared
	 */
	public void sharedFileUnsharedDirectory(String directory);

	/**
	 * Notifies listeners that the saving of the shared files has started.
	 * @param filename The name of the file the shared files are saved to
	 */
	public void sharedFileSavingStarted(String filename);

	/**
	 * Notifies listeners that the saving of the shared files has failed.
	 * @param filename The name of the file the shared files are saved to
	 * @param exception The exception that caused saving to fail
	 */
	public void sharedFileSavingFailed(String filename, Exception exception);

	public void sharedFileSavingComplete(String filename);

	public void sharedFileScanStarted();

	public void sharedFileScanFailed(Exception exception);

	public void sharedFileScanComplete();

	public void sharedFileDirectoryScanStarted(String directory);

	public void sharedFileDirectoryScanFailed(String directory, Exception exception);

	public void sharedFileDirectoryScanComplete(String directory);

	public void sharedFileHashingStarted(String filename);

	public void sharedFileHashingFailed(String filename, Exception exception);
	
	public void sharedFileHashingProgress(String filename, int currentBlock, int blockCount);

	public void sharedFileHashingComplete(String filename, String key, long filelength, long duration);

	public void sharedFileListChanged();

}