/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import de.todesbaum.freemule.application.FileStatus;
import de.todesbaum.freemule.application.FreenetFile;


/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:FileProgressBar.java 221 2006-03-06 14:46:49Z bombe $
 */
public class FileProgressBar extends JPanel implements TableCellRenderer {

	private enum Status {
		available, notAvailable, requesting, saving, error
	}

	private FreenetFile freenetFile;

	private Map<Status, Color> stateColors;
	private boolean showPercentage = true;
	private double roundness = 0.5;
	private Font percentageFont = new Font("Tahoma", 0, 12);

	public FileProgressBar() {
		stateColors = new HashMap<Status, Color>();
		stateColors.put(Status.available, new Color(0.0f, 0.7f, 0.0f));
		stateColors.put(Status.notAvailable, new Color(0.3f, 0.3f, 0.3f));
		stateColors.put(Status.requesting, new Color(0.0f, 0.0f, 1.0f));
		stateColors.put(Status.saving, new Color(0.55f, 0.55f, 0.25f));
		stateColors.put(Status.error, new Color(1.0f, 0.0f, 0.0f));
	}

	public void setRoundness(double roundness) {
		this.roundness = roundness;
	}

	public void setShowPercentage(boolean showPercentage) {
		this.showPercentage = showPercentage;
	}

	private void paintBar(Graphics g, int xLeft, int xRight, int height, Color color) {
		for (int y = 0; y < height; y++) {
			double f = Math.cos((y - height / 2.0d) * Math.PI / height) * (roundness) + (1 - roundness);
			Color roundColor = new Color((int) (color.getRed() * f), (int) (color.getGreen() * f), (int) (color.getBlue() * f));
			g.setColor(roundColor);
			g.drawLine(xLeft, y, xRight, y);
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		int width = getWidth();
		int height = getHeight();
		switch (freenetFile.getStatus()) {
		case activated:
		case deactivated:
			paintBlocks(g, width, height);
			break;
		case complete:
			paintBar(g, 0, width - 1, height, Color.GREEN);
			break;
		case error:
			paintBar(g, 0, width - 1, height, Color.RED);
		}
	}

	private void paintBlocks(Graphics g, int width, int height) {
		int totalBlocks = freenetFile.getBlockCount();
		int completeBlocks = 0;
		Status[] blockStatus = new Status[totalBlocks];
		for (int blockIndex = 0; blockIndex < totalBlocks; blockIndex++) {
			switch (freenetFile.getBlockStatus(blockIndex)) {
			case available:
				blockStatus[blockIndex] = Status.available;
				completeBlocks++;
				break;
			case requesting:
				blockStatus[blockIndex] = Status.requesting;
				break;
			case saving:
				blockStatus[blockIndex] = Status.saving;
				break;
			case notAvailable:
				blockStatus[blockIndex] = Status.notAvailable;
			}
			if (freenetFile.getStatus() == FileStatus.deactivated) {
				blockStatus[blockIndex] = (blockStatus[blockIndex] == Status.available) ? Status.error : blockStatus[blockIndex];
			}
		}
		if (totalBlocks > 0) {
			for (int x = 0; x < width; x++) {
				int block = (int) (((double) x / width) * totalBlocks);
				int newBlock = block;
				int nx = x;
				while (newBlock == block) {
					nx++;
					newBlock = (int) (((double) nx / width) * totalBlocks);
				}
				nx--;
				Color color = stateColors.get(blockStatus[block]);
				paintBar(g, x, nx, height, color);
				x = nx;
			}
			// g.setColor(Color.BLACK);
			// g.drawRect(0, 0, width - 1, height - 1);
			// completeBlocks = Math.min(requiredBlocks, completeBlocks);
			g.setColor(new Color(0, 192, 0));
			g.drawRect(1, 1, width * completeBlocks / totalBlocks - 1, 1);
			g.setColor(Color.BLACK);
			g.drawRect(0, 0, width * completeBlocks / totalBlocks, 3);
			if (showPercentage) {
				g.setFont(percentageFont);
				g.setColor(Color.WHITE);
				String text = (completeBlocks * 100 / totalBlocks) + "%";
				FontMetrics metrics = g.getFontMetrics();
				Rectangle2D textBounds = metrics.getStringBounds(text, g);
				g.drawString(text, (int) (width / 2 - textBounds.getWidth() / 2), (int) ((height - 1) / 2 + textBounds.getHeight() / 2) - 1);
			}
		} else {
			paintBar(g, 0, width - 1, height, stateColors.get(Status.notAvailable));
			if (showPercentage) {
				g.setFont(percentageFont);
				g.setColor(Color.WHITE);
				String text = "0%";
				FontMetrics metrics = g.getFontMetrics();
				Rectangle2D textBounds = metrics.getStringBounds(text, g);
				g.drawString(text, (int) (width / 2 - textBounds.getWidth() / 2), (int) ((height - 1) / 2 + textBounds.getHeight() / 2) - 1);
			}
		}
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(super.getPreferredSize().width, 21);
	}

	/**
	 * {@inheritDoc}
	 */
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		freenetFile = (FreenetFile) value;
		return this;
	}
}
