/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.gui;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import de.todesbaum.freemule.application.SharedDirectory;


/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id:SharedDirectoriesModel.java 221 2006-03-06 14:46:49Z  $
 */
public class SharedDirectoriesModel implements TableModel {

	private static final String[] COLUMN_NAMES = new String[] { "Directory", "Recursive" };
	private static final Class[] COLUMN_CLASSES = new Class[] { String.class, Boolean.class };

	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();
	private List<String> directories = new ArrayList<String>();
	private List<Boolean> recursives = new ArrayList<Boolean>();
	
	public synchronized void setSharedDirectories(SharedDirectory[] sharedDirectories) {
		this.directories.clear();
		this.recursives.clear();
		for (SharedDirectory sharedDirectory: sharedDirectories) {
			directories.add(sharedDirectory.getDirectory());
			recursives.add(sharedDirectory.isRecursive());
		}
	}
	
	public synchronized SharedDirectory[] getSharedDirectories() {
		SharedDirectory[] result = new SharedDirectory[directories.size()];
		for (int directoryIndex = 0, directoryCount = directories.size(); directoryIndex < directoryCount; directoryIndex++) {
			result[directoryIndex] = new SharedDirectory(directories.get(directoryIndex), recursives.get(directoryIndex));
		}
		return result;
	}
	
	public synchronized void addSharedDirectory(String sharedDirectory, boolean recursive) {
		directories.add(sharedDirectory);
		recursives.add(recursive);
		fireRowAdded(directories.size() - 1);
	}
	
	public synchronized void deleteSharedDirectory(int index) {
		directories.remove(index);
		recursives.remove(index);
		fireRowDeleted(index);
	}
	
	public synchronized void setSharedDirectory(int index, String sharedDirectory) {
		directories.set(index, sharedDirectory);
		fireRowUpdated(index);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int getColumnCount() {
		return COLUMN_CLASSES.length;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}

	/**
	 * {@inheritDoc}
	 */
	public Class< ? > getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int getRowCount() {
		return directories.size();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Object getValueAt(int row, int column) {
		switch (column) {
		case 0:
			return directories.get(row);
		case 1:
			return recursives.get(row);
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case 0:
			directories.set(rowIndex, (String) aValue);
			break;
		case 1:
			recursives.set(rowIndex, (Boolean) aValue);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void addTableModelListener(TableModelListener tableModelListener) {
		listeners.add(tableModelListener);
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeTableModelListener(TableModelListener tableModelListener) {
		listeners.remove(tableModelListener);
	}
	
	private void fireEvent(TableModelEvent tableModelEvent) {
		for (TableModelListener tableModelListener: listeners) {
			tableModelListener.tableChanged(tableModelEvent);
		}
	}
	
	private void fireRowAdded(int row) {
		TableModelEvent tableModelEvent = new TableModelEvent(this, row, row, 0, TableModelEvent.INSERT);
		fireEvent(tableModelEvent);
	}

	private void fireRowUpdated(int row) {
		TableModelEvent tableModelEvent = new TableModelEvent(this, row, row, 0, TableModelEvent.UPDATE);
		fireEvent(tableModelEvent);
	}

	private void fireRowDeleted(int row) {
		TableModelEvent tableModelEvent = new TableModelEvent(this, row, row, 0, TableModelEvent.DELETE);
		fireEvent(tableModelEvent);
	}

}
