/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;

/**
 * An {@link OutputStream} that writes to a part of a file specified by an
 * offset and a length.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:FilePartOutputStream.java 221 2006-03-06 14:46:49Z bombe $
 */
public class FilePartOutputStream extends OutputStream {

	/**
	 * The underlying file to write to.
	 */
	private RandomAccessFile randomAccessFile;

	/**
	 * The number of bytes that may still be written.
	 */
	private long length;

	/**
	 * Creates a new FilePartOutputStream that restricts writing to the part
	 * specified by <code>offset</code> and <code>length</code>.
	 * 
	 * @param filename
	 *            The name of the file to write to
	 * @param offset
	 *            The offset to begin writing at
	 * @param length
	 *            The maximum number of bytes to write
	 * @throws FileNotFoundException
	 *             if the file can not be created
	 * @throws IOException
	 *             if an I/O error occurs
	 */
	public FilePartOutputStream(String filename, long offset, long length) throws FileNotFoundException, IOException {
		this(new File(filename), offset, length);
	}

	/**
	 * Creates a new FilePartOutputStream that restricts writing to the part
	 * specified by <code>offset</code> and <code>length</code>.
	 * 
	 * @param file
	 *            The file to write to
	 * @param offset
	 *            The offset to begin writing at
	 * @param length
	 *            The maximum number of bytes to write
	 * @throws FileNotFoundException
	 *             if the file can not be created
	 * @throws IOException
	 *             if an I/O error occurs
	 */
	public FilePartOutputStream(File file, long offset, long length) throws FileNotFoundException, IOException {
		randomAccessFile = new RandomAccessFile(file, "rw");
		randomAccessFile.seek(offset);
		this.length = length;
	}

	/**
	 * Closes this stream and releases all used resources.
	 */
	@Override
	public void close() throws IOException {
		if (randomAccessFile != null) {
			randomAccessFile.close();
		}
		randomAccessFile = null;
	}

	/**
	 * Writes a single byte to the stream.
	 * 
	 * @param data
	 *            The byte to write
	 * @throws IOException
	 *             if the write limit has been reached or another I/O error
	 *             occurs
	 */
	@Override
	public void write(int data) throws IOException {
		if (length <= 0) {
			throw new IOException("write limit reached");
		}
		randomAccessFile.write(data);
		length--;
	}

	/**
	 * Writes a number of bytes to the stream. Bytes are written to the stream
	 * until either the buffer is depleted or the write limit has been reached.
	 * In the latter case an IOException will only be thrown if the write limit
	 * was reached before a single byte was written. Calling this method is
	 * equivalent to calling
	 * {@link #write(byte[], int, int) write(data, 0, data.length)}.
	 * 
	 * @param data
	 *            The bytes to write
	 * @throws IOException
	 *             if the write limit has been reached or another I/O error
	 *             occurs.
	 */
	@Override
	public void write(byte[] data) throws IOException {
		write(data, 0, data.length);
	}

	/**
	 * Writes a number of bytes to the stream. Bytes are written to the stream
	 * until either the buffer is depleted or the write limit has been reached.
	 * In the latter case an IOException will only be thrown if the write limit
	 * was reached before a single byte was written.
	 * 
	 * @param data
	 *            The bytes to write
	 * @param off
	 *            The offset at which to begin reading the buffer
	 * @param len
	 *            The number of bytes to read from the buffer
	 * @throws IOException
	 *             if the write limit has been reached or another I/O error
	 *             occurs.
	 */
	@Override
	public void write(byte[] data, int off, int len) throws IOException {
		if (length <= 0) {
			throw new IOException("write limit reached");
		}
		randomAccessFile.write(data, off, (int) Math.min(Integer.MAX_VALUE, Math.min(length, len)));
		length -= Math.min(length, len);
	}

}
