/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.util;

/**
 * Helper class to format size or length numbers with size suffixes. A number of <code>1048576</code> (<code>2^20</code>) would be formatted
 * into <code>1 MiB</code>, <code>1048575</code> (<code>2^20 - 1</code>) would be formatted into <code>1023 KiB</code>.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:NumberFormatter.java 221 2006-03-06 14:46:49Z bombe $
 */
public class NumberFormatter {

	/**
	 * Does nothing. As the methods of this class are static there is no need ever to call this constructor.
	 */
	private NumberFormatter() {
	}

	/**
	 * The abbreviated suffixes for numbers up to 10^15.
	 */
	private static final String[] units = { "Bytes", "KiB", "MiB", "GiB", "TiB", "PiB" };

	/**
	 * Formats a number so that it has a maximum of four digits and an appropriate suffix.
	 * 
	 * @param number
	 *            The number to format
	 * @return The formatted number
	 */
	public static String format(long number) {
		double dotted = number * 10;
		int unit = 0;
		while (String.valueOf((int) dotted).length() > 4) {
			dotted = (int) (dotted / 1024);
			unit++;
		}
		dotted = dotted / 10.0;
		return String.valueOf(dotted) + " " + units[unit];
	}

	/**
	 * Zero-pads a number up to the specified number of digits.
	 * 
	 * @param number
	 *            The number to zero-pad
	 * @param digits
	 *            The number of digits for the number
	 * @return The zero-padded number
	 */
	public static String nullify(int number, int digits) {
		String text = String.valueOf(number);
		while (text.length() < digits) {
			text = "0" + text;
		}
		return text;
	}

}
