/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.util;

import java.util.ArrayList;
import java.util.List;

/**
 * Contains some methods that deal with strings.
 * 
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id:StringUtils.java 221 2006-03-06 14:46:49Z  $
 */
public class StringUtils {

	/**
	 * Splits <code>line</code> into words and returns those words. Words are separated by whitespace (as determined by
	 * {@link Character#isWhitespace(char)}) and are optionally delimited by quotes. Quotes are mandatory if a word contains whitespace.
	 * 
	 * @param line
	 *            The line to split into words
	 * @return The words of this line
	 */
	public static String[] splitLineIntoWords(String line) {
		List<String> words = new ArrayList<String>();
		boolean inWord = false;
		boolean inQuotes = false;
		boolean inBackslash = false;
		int wordStart = 0;
		int wordEnd = -1;
		int index = 0;
		for (; index < line.length(); index++) {
			char currentChar = line.charAt(index);
			if (inBackslash) {
				inBackslash = !inBackslash;
				if (currentChar == 'n') {
					line = line.substring(0, index) + "\n" + line.substring(index + 1);
				}
				continue;
			}
			if (currentChar == '\\') {
				inBackslash = true;
				line = line.substring(0, index) + line.substring(index + 1);
				continue;
			}
			if (currentChar == '"') {
				inQuotes = !inQuotes;
				if (!inWord) {
					wordStart = index + 1;
				} else {
					wordEnd = index;
				}
			} else if (Character.isWhitespace(currentChar)) {
				if (inWord) {
					if (!inQuotes) {
						if (wordEnd == -1) {
							wordEnd = index;
						}
						String word = line.substring(wordStart, wordEnd);
						words.add(word);
						inWord = false;
						wordEnd = -1;
					}
				}
			} else {
				if (!inWord) {
					inWord = true;
					wordStart = index;
				}
			}
		}
		if (inWord) {
			if (wordEnd == -1) {
				wordEnd = index;
			}
			String word = line.substring(wordStart, index);
			words.add(word);
		}
		return words.toArray(new String[words.size()]);
	}

	/**
	 * Encloses the words in quotes, if necessary, and appends them using a single space character (\\u0020) into a line.
	 * 
	 * @param words
	 *            The words to format and concatenate
	 * @return The line
	 */
	public static String formatWords(String... words) {
		StringBuffer result = new StringBuffer();
		for (String word: words) {
			word = word.replace("\\", "\\\\");
			word = word.replace("\n", "\\n");
			word = word.replace("\"", "\\\"");
			if (result.length() > 0) {
				result.append(" ");
			}
			if (word.indexOf(" ") > -1) {
				result.append("\"" + word + "\"");
			} else {
				result.append(word);
			}
		}
		return result.toString();
	}

}
