/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

/**
 * This class represents a Freenet Client. It is the main class used to
 * communicate with the {@link FcpNode Freenet node}.
 * <p>
 * Even though an FCP connection is torn down after a single command, this
 * client implementation does not have to be reset or re-created after each
 * command. Simply create an instance for one node and use it over and over
 * again. The implementation should be thread-safe but has not been specifically
 * tested for it. Yes, I am lazy.
 * 
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpClient.java 218 2006-03-06 14:43:35Z  $
 */
public class FcpClient {

	/**
	 * The Freenet node to communicate with.
	 */
	private FcpNode fcpNode;

	/**
	 * Creates a new FCP Client for the specified Freenet node.
	 * 
	 * @param fcpNode
	 *            The Freenet node to communicate with
	 */
	public FcpClient(FcpNode fcpNode) {
		this.fcpNode = fcpNode;
	}

	/**
	 * Executes an FCP command. This method handles the necessary connection
	 * setup, both physical and logical. The command to execute has to be
	 * created with the necessary parameters before using this method.
	 * 
	 * @param fcpCommand
	 *            The FCP command to execute
	 * @throws FcpFormatErrorException
	 *             if the Freenet node responds with a
	 *             <code>"FormatError"</code>
	 * @throws FcpFailedException
	 *             if the FCP command failed to execute
	 * @throws IOException
	 *             if an I/O error occurs
	 */
	public void execute(FcpCommand fcpCommand) throws FcpException, IOException {
		Socket fcpSocket = null;
		InputStream fcpInput = null;
		OutputStream fcpOutput = null;
		try {
			fcpSocket = new Socket();
			fcpSocket.setKeepAlive(true);
			fcpSocket.connect(new InetSocketAddress(fcpNode.getHost(), fcpNode.getPort()));
			fcpInput = fcpSocket.getInputStream();
			fcpOutput = fcpSocket.getOutputStream();

			fcpOutput.write(new byte[] { 0x00, 0x00, 0x00, 0x02 });

			fcpOutput.write((fcpCommand.getName() + "\r\n").getBytes());
			if (fcpCommand.writeCommand(fcpOutput)) {
				fcpOutput.write("EndMessage\r\n".getBytes());
			}
			fcpOutput.flush();

			FcpMessage answer = null;
			do {
				answer = FcpMessage.readMessage(fcpInput);
				if ("Failed".equals(answer.getName())) {
					throw new FcpFailedException(answer, answer.getParameter("Reason"));
				}
				if ("FormatError".equals(answer.getName())) {
					throw new FcpFormatErrorException(answer, answer.getParameter("Reason"));
				}
				fcpCommand.handleFcpMessage(answer);
			} while (!fcpCommand.isComplete());
		} finally {
			try {
				if (fcpOutput != null) {
					fcpOutput.close();
				}
			} catch (IOException e) {
			}
			try {
				if (fcpInput != null) {
					fcpInput.close();
				}
			} catch (IOException e) {
			}
			try {
				if (fcpSocket != null) {
					fcpSocket.close();
				}
			} catch (IOException e) {
			}
		}
	}

}
