/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.io.IOException;
import java.io.OutputStream;

/**
 * FCP <code>ClientGet</code> command implementation. Create a new instance
 * with {@link #FcpClientGet(String, int)} giving the URI and the Hops to live
 * value, execute it via {@link FcpClient#execute(FcpCommand)} and read the
 * results via {@link #getData()} and {@link #getMetadata()}.
 * 
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpClientGet.java 218 2006-03-06 14:43:35Z  $
 */
public class FcpClientGet extends FcpCommand {

	/**
	 * The URI to get.
	 */
	private String uri;

	/**
	 * The number of hops the request will survive in the net.
	 */
	private int hopsToLive;

	/**
	 * Whether to skip the local data store in search for this key.
	 */
	private boolean removeLocalKey;

	/**
	 * The retrieved data.
	 */
	private byte[] data;

	/**
	 * The retrieved metadata.
	 */
	private byte[] metadata;

	/**
	 * The length of the retrieved data.
	 */
	private int dataLength;

	/**
	 * The length of the retrieved metadata.
	 */
	private int metadataLength;

	/**
	 * Internally used pointer to offset of the next chunk of data.
	 */
	private int dataPointer;

	/**
	 * Creates a new FCP <code>ClientGet</code> command with the specified
	 * parameters.
	 * 
	 * @param uri
	 *            The key of the data
	 * @param hopsToLive
	 *            The number of hops to live
	 */
	public FcpClientGet(String uri, int hopsToLive) {
		this(uri, hopsToLive, false);
	}

	/**
	 * Creates a new FCP <code>GET</code> command with the specified
	 * parameters.
	 * 
	 * @param uri
	 *            The key of the data
	 * @param hopsToLive
	 *            The number of hops to live
	 * @param removeLocalKey
	 *            <code>true</code> if the local data store is to be skipped
	 *            in search of the data
	 */
	public FcpClientGet(String uri, int hopsToLive, boolean removeLocalKey) {
		this.uri = uri;
		this.hopsToLive = hopsToLive;
		this.removeLocalKey = removeLocalKey;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return "ClientGet";
	}

	/**
	 * Returns the retrieved data.
	 * 
	 * @return The retrieved data
	 */
	public byte[] getData() {
		byte[] result = new byte[data.length];
		System.arraycopy(data, 0, result, 0, data.length);
		return result;
	}

	/**
	 * Returns the length of the retrieved data. Is the same as
	 * <code>getData().length</code>.
	 * 
	 * @return The length of the retrieved data
	 */
	public int getDataLength() {
		return dataLength;
	}

	/**
	 * Returns the retrieved metadata.
	 * 
	 * @return The retrieved metadata
	 */
	public byte[] getMetadata() {
		byte[] result = new byte[metadata.length];
		System.arraycopy(metadata, 0, result, 0, metadata.length);
		return result;
	}

	/**
	 * Returns the length of the retrieved metadata. Is the same as
	 * <code>getMetadata().length</code>.
	 * 
	 * @return The length of the retrieved metadata.
	 */
	public int getMetadataLength() {
		return metadataLength;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		outputStream.write(("URI=" + uri + "\r\n").getBytes());
		outputStream.write(("HopsToLive=" + hopsToLive + "\r\n").getBytes());
		outputStream.write(("RemoveLocalKey=" + removeLocalKey + "\r\n").getBytes());
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) throws FcpException {
		String fcpMessageName = fcpMessage.getName();
		if (fcpMessageName.equals("DataFound")) {
			metadataLength = (int) parseHex(fcpMessage.getParameter("MetadataLength"));
			metadata = new byte[metadataLength];
			dataLength = (int) parseHex(fcpMessage.getParameter("DataLength")) - metadataLength;
			data = new byte[dataLength];
			dataPointer = 0;
		} else if (fcpMessageName.equals("Restarted")) {
			fireFcpEvent(fcpMessageName + ": " + fcpMessage.getParameter("Reason"));
			dataPointer = 0;
		} else if (fcpMessageName.equals("URIError")) {
			throw new FcpURIErrorException(fcpMessage, fcpMessage.getParameter("Reason"));
		} else if (fcpMessageName.equals("DataNotFound")) {
			throw new FcpDataNotFoundException(fcpMessage);
		} else if (fcpMessageName.equals("RouteNotFound")) {
			throw new FcpRouteNotFoundException(fcpMessage);
		} else if (fcpMessageName.equals("DataChunk")) {
			int length = (int) parseHex(fcpMessage.getParameter("Length"));
			if (dataPointer < metadataLength) {
				if (metadataLength - dataPointer > fcpMessage.getData().length) {
					System.arraycopy(fcpMessage.getData(), 0, metadata, dataPointer, Math.min(fcpMessage.getData().length, metadataLength - dataPointer));
				} else {
					System.arraycopy(fcpMessage.getData(), 0, metadata, dataPointer, metadataLength - dataPointer);
					System.arraycopy(fcpMessage.getData(), metadataLength - dataPointer, data, 0, length - (metadataLength - dataPointer));
				}
			} else {
				System.arraycopy(fcpMessage.getData(), 0, data, dataPointer - metadataLength, length);
			}
			dataPointer += length;
			complete = dataPointer == (metadataLength + dataLength);
		}
	}

}
