/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.io.IOException;
import java.io.OutputStream;

/**
 * FCP <code>ClientInfo</code> command implementation. The ClientInfo command
 * is used to get some detailed information about the Freenet node.
 * 
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpClientInfo.java 218 2006-03-06 14:43:35Z  $
 */
public class FcpClientInfo extends FcpCommand {

	/**
	 * The architecture (e.g. 386) the node is running on.
	 */
	private String architecture;

	/**
	 * The operating system (e.g. <code>Windows 2000</code> or
	 * <code>Linux</code>) the node is running on.
	 */
	private String operatingSystem;

	/**
	 * The version of the operating system (e.g. <code>2.6.11</code>) the
	 * node is running on.
	 */
	private String operatingSystemVersion;

	/**
	 * The manufacturer of the java virtual machine (e.g.
	 * <code>SUN Microsystems, Inc.</code>) the node is running on.
	 */
	private String javaVendor;

	/**
	 * The name of the java virtual machine the node is running on.
	 */
	private String javaName;

	/**
	 * The version of the java virtual machine the node is running on.
	 */
	private String javaVersion;

	/**
	 * The amount of memory the java virtual machine has allocated.
	 */
	private int allocatedMemory;

	/**
	 * The amount of free memory within the allocated memory.
	 */
	private int freeMemory;

	/**
	 * The maximum size of the data store.
	 */
	private long datastoreMax;

	/**
	 * The amount of bytes used in the data store.
	 */
	private long datastoreUsed;

	/**
	 * The amount of bytes unused in the data store.
	 */
	private long datastoreFree;

	/**
	 * The maximum file size the node can handle.
	 */
	private int maxFileSize;

	/**
	 * The most recent timestamp in the data store.
	 */
	private long mostRecentTimestamp;

	/**
	 * The least recent (i.e. oldest) timestamp in the data store.
	 */
	private long leastRecentTimestamp;

	/**
	 * The current routing time of the node.
	 */
	private int routingTime;

	/**
	 * The number of available threads in the node.
	 */
	private int availableThreads;

	/**
	 * The number of active jobs in the node.
	 */
	private int activeJobs;
	private int nodePort;
	private String nodeAddress;
	private int estimatedLoad;
	private boolean isTransient;
	private int processors;
	private int estimatedRateLimitingLoad;

	/**
	 * @see de.todesbaum.util.freenet.fcp.FcpCommand#getName()
	 */
	public String getName() {
		return "ClientInfo";
	}

	/**
	 * @return Returns the activeJobs.
	 */
	public int getActiveJobs() {
		return activeJobs;
	}

	/**
	 * @return Returns the allocatedMemory.
	 */
	public int getAllocatedMemory() {
		return allocatedMemory;
	}

	/**
	 * @return Returns the architecture.
	 */
	public String getArchitecture() {
		return architecture;
	}

	/**
	 * @return Returns the availableThreads.
	 */
	public int getAvailableThreads() {
		return availableThreads;
	}

	/**
	 * @return Returns the datastoreFree.
	 */
	public long getDatastoreFree() {
		return datastoreFree;
	}

	/**
	 * @return Returns the datastoreMax.
	 */
	public long getDatastoreMax() {
		return datastoreMax;
	}

	/**
	 * @return Returns the datastoreUsed.
	 */
	public long getDatastoreUsed() {
		return datastoreUsed;
	}

	/**
	 * @return Returns the estimatedLoad.
	 */
	public int getEstimatedLoad() {
		return estimatedLoad;
	}

	/**
	 * @return Returns the freeMemory.
	 */
	public int getFreeMemory() {
		return freeMemory;
	}

	/**
	 * @return Returns the istransient.
	 */
	public boolean isTransient() {
		return isTransient;
	}

	/**
	 * @return Returns the javaName.
	 */
	public String getJavaName() {
		return javaName;
	}

	/**
	 * @return Returns the javaVendor.
	 */
	public String getJavaVendor() {
		return javaVendor;
	}

	/**
	 * @return Returns the javaVersion.
	 */
	public String getJavaVersion() {
		return javaVersion;
	}

	/**
	 * @return Returns the leastRecentTimestamp.
	 */
	public long getLeastRecentTimestamp() {
		return leastRecentTimestamp;
	}

	/**
	 * @return Returns the maxFileSize.
	 */
	public int getMaxFileSize() {
		return maxFileSize;
	}

	/**
	 * @return Returns the mostRecentTimestamp.
	 */
	public long getMostRecentTimestamp() {
		return mostRecentTimestamp;
	}

	/**
	 * @return Returns the nodeAddress.
	 */
	public String getNodeAddress() {
		return nodeAddress;
	}

	/**
	 * @return Returns the nodePort.
	 */
	public int getNodePort() {
		return nodePort;
	}

	/**
	 * @return Returns the operatingSystem.
	 */
	public String getOperatingSystem() {
		return operatingSystem;
	}

	/**
	 * @return Returns the operatingSystemVersion.
	 */
	public String getOperatingSystemVersion() {
		return operatingSystemVersion;
	}

	/**
	 * @return Returns the routingTime.
	 */
	public int getRoutingTime() {
		return routingTime;
	}

	/**
	 * @return Returns the estimatedRateLimitingLoad.
	 */
	public int getEstimatedRateLimitingLoad() {
		return estimatedRateLimitingLoad;
	}

	/**
	 * @return Returns the processors.
	 */
	public int getProcessors() {
		return processors;
	}

	/**
	 * @see de.todesbaum.util.freenet.fcp.FcpCommand#writeCommand(java.io.OutputStream)
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		return true;
	}

	/**
	 * @see de.todesbaum.util.freenet.fcp.FcpCommand#handleFcpMessage(de.todesbaum.util.freenet.fcp.FcpMessage)
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) {
		if (fcpMessage.getName().equals("NodeInfo")) {
			architecture = fcpMessage.getParameter("Architecture");
			operatingSystem = fcpMessage.getParameter("OperatingSystem");
			operatingSystemVersion = fcpMessage.getParameter("OperatingSystemVersion");
			javaVendor = fcpMessage.getParameter("JavaVendor");
			javaName = fcpMessage.getParameter("JavaName");
			javaVersion = fcpMessage.getParameter("JavaVersion");
			allocatedMemory = (int) parseHex(fcpMessage.getParameter("AllocatedMemory"));
			freeMemory = (int) parseHex(fcpMessage.getParameter("FreeMemory"));
			datastoreMax = parseHex(fcpMessage.getParameter("DatastoreMax"));
			datastoreUsed = parseHex(fcpMessage.getParameter("DatastoreUsed"));
			datastoreFree = parseHex(fcpMessage.getParameter("DatastoreFree"));
			maxFileSize = (int) parseHex(fcpMessage.getParameter("MaxFileSize"));
			mostRecentTimestamp = parseHex(fcpMessage.getParameter("MostRecentTimestamp"));
			leastRecentTimestamp = parseHex(fcpMessage.getParameter("LeastRecentTimestamp"));
			routingTime = (int) parseHex(fcpMessage.getParameter("RoutingTime"));
			availableThreads = (int) parseHex(fcpMessage.getParameter("AvailableThreads"));
			activeJobs = (int) parseHex(fcpMessage.getParameter("ActiveJobs"));
			nodePort = (int) parseHex(fcpMessage.getParameter("NodePort"));
			nodeAddress = fcpMessage.getParameter("NodeAddress");
			estimatedLoad = (int) parseHex(fcpMessage.getParameter("EstimatedLoad"));
			isTransient = Boolean.valueOf(fcpMessage.getParameter("IsTransient")).booleanValue();
			processors = (int) parseHex(fcpMessage.getParameter("Processors"));
			estimatedRateLimitingLoad = (int) parseHex(fcpMessage.getParameter("EstimatedRateLimitingLoad"));
			complete = true;
		}
	}

}
