/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.SequenceInputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

import junit.framework.TestCase;

/**
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpClientTest.java 218 2006-03-06 14:43:35Z  $
 */
public class FcpClientTest extends TestCase {

	private FcpNode fcpNode;
	private FcpClient fcpClient;

	private byte[] gplText;

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		fcpNode = new FcpNode("127.0.0.1", 8481);
		fcpClient = new FcpClient(fcpNode);
		FileInputStream fileInput = new FileInputStream("COPYING");
		ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
		int b;
		while ((b = fileInput.read()) != -1) {
			byteOutput.write(b);
		}
		fileInput.close();
		byteOutput.close();
		gplText = byteOutput.toByteArray();
	}

	public void testClientHello() throws FcpException, IOException {
		FcpClientHello fcpClientHello = new FcpClientHello();
		fcpClient.execute(fcpClientHello);
		assertEquals("1.2", fcpClientHello.getProtocol());
	}

	public void testClientInfo() throws FcpException, IOException {
		FcpClientInfo fcpClientInfo = new FcpClientInfo();
		fcpClient.execute(fcpClientInfo);
		assertEquals(false, fcpClientInfo.isTransient());
	}

	public void _testClientGet1() throws FcpException, IOException {
		FcpClientGet fcpClientGet = new FcpClientGet("CHK@6ob-y0GkE3P11axo61UcB0WSD28SAwI,OqU13PcsdBEp1eurTYopbg", 25);
		fcpClient.execute(fcpClientGet);
		assertEquals(186426, fcpClientGet.getDataLength());
		assertTrue(fcpClientGet.isComplete());
	}

	public void _testClientGet2() throws IOException {
		FcpClientGet fcpClientGet = new FcpClientGet("CHK@6ob-y0GkE3P11bxo61UcB0WSD28SAwI,OqU13PcsdBEp1eurTYopbg", 25);
		try {
			fcpClient.execute(fcpClientGet);
			assertEquals(true, fcpClientGet.isComplete());
		} catch (FcpException fcpe1) {
			assertEquals(FcpDataNotFoundException.class.getName(), fcpe1.getClass().getName());
			assertEquals(false, fcpClientGet.isComplete());
		}
	}

	public void _testClientPut1() throws IOException {
		FcpClientPut fcpClientPut = new FcpClientPut("CHK@", 10, true, gplText);
		try {
			fcpClient.execute(fcpClientPut);
			assertEquals("freenet:CHK@O67b08gNCJog1WpXT3-wuFS4ol0OAwI,ylyWL6W4-Xg5JgA-gxTGZg", fcpClientPut.getFinalURI());
			assertEquals(true, fcpClientPut.isComplete());
		} catch (FcpException fcpe1) {
			if (fcpe1 instanceof FcpKeyCollisionException) {
				assertEquals(true, fcpClientPut.isComplete());
			} else {
				assertEquals(false, fcpClientPut.isComplete());
			}
		}
	}

	public void _testClientPut2() throws IOException {
		FcpClientPut fcpClientPut = new FcpClientPut("KSK@COPYING", 10, true, gplText);
		try {
			fcpClient.execute(fcpClientPut);
			assertEquals("freenet:KSK@COPYING", fcpClientPut.getFinalURI());
			assertEquals(true, fcpClientPut.isComplete());
		} catch (FcpException fcpe1) {
			if (fcpe1 instanceof FcpKeyCollisionException) {
				assertEquals(true, fcpClientPut.isComplete());
			} else {
				assertEquals(false, fcpClientPut.isComplete());
			}
		}
	}

	public void _testClientPut3() throws IOException {
		FcpClientPut fcpClientPut = new FcpClientPut("KSK@COPYING/2", 10, true, gplText, new byte[0]);
		try {
			fcpClient.execute(fcpClientPut);
			assertEquals("freenet:KSK@COPYING/2", fcpClientPut.getFinalURI());
			assertEquals(true, fcpClientPut.isComplete());
		} catch (FcpException fcpe1) {
			if (fcpe1 instanceof FcpKeyCollisionException) {
				assertEquals(true, fcpClientPut.isComplete());
			} else {
				assertEquals(false, fcpClientPut.isComplete());
			}
		}
	}

	public void testGenerateCHK() throws IOException {
		FcpGenerateCHK generateCHK = new FcpGenerateCHK(gplText);
		try {
			fcpClient.execute(generateCHK);
			assertEquals(true, generateCHK.isComplete());
			assertEquals("freenet:CHK@O67b08gNCJog1WpXT3-wuFS4ol0OAwI,ylyWL6W4-Xg5JgA-gxTGZg", generateCHK.getURI());
		} catch (FcpException fcpe1) {
			assertEquals(false, generateCHK.isComplete());
		}
	}

	public void testGenerateCHK2() throws Exception {
		FcpGenerateCHK generateCHK = new FcpGenerateCHK(new byte[0], new byte[0]);
		try {
			fcpClient.execute(generateCHK);
			assertEquals(true, generateCHK.isComplete());
			System.out.println(generateCHK.getURI());
		} catch (FcpException fcpe1) {
			assertEquals(false, generateCHK.isComplete());
			throw fcpe1;
		}
	}

	public void testGenerateSVKPair() throws IOException {
		FcpGenerateSVKPair generateSVKPair = new FcpGenerateSVKPair();
		try {
			fcpClient.execute(generateSVKPair);
			assertEquals(true, generateSVKPair.isComplete());
		} catch (FcpException fcpe1) {
			assertEquals(false, generateSVKPair.isComplete());
		}
	}

	public void testInvertPrivateKey() throws IOException {
		FcpInvertPrivateKey invertPrivateKey = new FcpInvertPrivateKey("Fgl4aHSndYiH5TxIgb1jE2k5hIs");
		try {
			fcpClient.execute(invertPrivateKey);
			assertEquals(true, invertPrivateKey.isComplete());
			assertEquals("Bmhn~P6qHFCNQg628cdb649voCA" + "PAgM", invertPrivateKey.getPublicKey());
		} catch (FcpException fcpe1) {
			assertEquals(false, invertPrivateKey.isComplete());
		}
	}

	public void _testFecEncode() throws FcpException, IOException {
		FcpFecSegmentFile segmentFile = new FcpFecSegmentFile(13051904);
		try {
			fcpClient.execute(segmentFile);
			assertEquals(1, segmentFile.getSegmentHeaderCount());
			SegmentHeader segmentHeader = segmentFile.getSegmentHeader(0);
			assertEquals(50, segmentHeader.getBlockCount());
			assertEquals(262144, segmentHeader.getBlockSize());
			assertEquals(50, segmentHeader.getBlocksRequired());
			assertEquals(25, segmentHeader.getCheckBlockCount());
			assertEquals(262144, segmentHeader.getCheckBlockSize());
			assertEquals("OnionFEC_a_1_2", segmentHeader.getFecAlgorithm());
			assertEquals(13051904, segmentHeader.getFileLength());
			assertEquals(0, segmentHeader.getOffset());
			assertEquals(0, segmentHeader.getSegment());
			assertEquals(1, segmentHeader.getSegmentCount());
			
			Random rnd = new Random(0);
			byte[] data = new byte[13051904];
			rnd.nextBytes(data);
			FcpFecEncodeSegment encodeSegment = new FcpFecEncodeSegment(segmentHeader, data);
			System.out.println("Encoding Segment...");
			fcpClient.execute(encodeSegment);
			
			assertEquals(25, encodeSegment.getCheckBlockCount());
			assertEquals(262144, encodeSegment.getCheckBlockSize());
			
			BlockMap blockMap = new BlockMap(segmentHeader.getBlockCount(), segmentHeader.getCheckBlockCount());
//			String[] checkBlockURIs = new String[] {
//					"freenet:CHK@i4o-bKU2lGV0xigaRdnOu9T~nkgSAwI,h5NSlRdUm5pPzeTLaiF9zw",
//					"freenet:CHK@DsvK0icOVUPV2qfoMjRLfT6ixHQSAwI,J5wT1dffCeQMd4OO5sTzQA",
//					"freenet:CHK@eNmx46oeoWUQAEtuZxywOJweLX0SAwI,lGyDyaD56Q0QWDLDvubdMw",
//					"freenet:CHK@gWr9DPzVXNuAyUYaEJdIDGe2YHUSAwI,HS7-oXW0~FsA8HmFFqAvig",
//					"freenet:CHK@uW8xiiq8IeUyQhNuM1bK3xB6S1wSAwI,mwfwQlcAykN5u5fHYaxivw",
//					"freenet:CHK@tl3IGuxUI82vXJCH~uJZDvx73OUSAwI,FANCNBW~PxAWWXYCMM3DEg",
//					"freenet:CHK@~jJp53t4CxkVCkV~JG1zS88ZFDUSAwI,-c-yjncZYMpCazW9ZrKO9A",
//					"freenet:CHK@QL7oqS8ZpSpZ8ylMo1t-Dd5Kja4SAwI,1cJ8xkhPqhM8Br1VmUyYgQ",
//					"freenet:CHK@B9CkQHFsXRW1XEuwpyS~zxC~qfYSAwI,6bXI~CEbTkpP0eHctSVGhA",
//					"freenet:CHK@sGyBGSrKhrpJoN6YcwogW~Q-4XsSAwI,cksiVvznO6m8JSy4~QWJjw",
//					"freenet:CHK@I5qDeAV30tRaZFnsQTgsBsPlOPoSAwI,93Y8-1MuKYxiBQGRPvP~Qg",
//					"freenet:CHK@gIqXsUfh1uIIjbiMEn1nxu6xgYMSAwI,9Dwf2tHCDgmSNGR5bXXmdg",
//					"freenet:CHK@gYtTHw-fDGeGY0E95FB76UtXDtsSAwI,Iw1i8b7lcpuUiUAEaiPNUw",
//					"freenet:CHK@RXqvpgZlMT3FTT7R5RAIrdLH04ASAwI,jd3ScVtvtn9t-Cjxaol1WA",
//					"freenet:CHK@SS0scH4mw-FdSNLSm4N517AasKASAwI,7RmNn9sMrDbXJzCDCzw5zg",
//					"freenet:CHK@xLgPysNFGGBvKDaqe3c4Kz9sBoESAwI,K60tTQipAPuG1GIIUGoywQ",
//					"freenet:CHK@NOZb6~FZ~a07ysDnl9dMnzIrk6ESAwI,V5akpXrKMQrRdfgdxPuTrA",
//					"freenet:CHK@hm6w3~7KzuTtUmIaiRc~xtC5HvcSAwI,F9hRHUU1031bTyN5fa2aOQ",
//					"freenet:CHK@I4s71RI5ZnjIez3hqvOJSPUmyCMSAwI,O3WJ6X-g-UyCdGR4LVerXg",
//					"freenet:CHK@Y6LNbzivVoLYC-UViKoFULq4i7cSAwI,Aa0ZwG138RsHWwHGRhHwsA",
//					"freenet:CHK@apaOIvVKQ63vnM-uha8U6tdcaJQSAwI,MKML1Ttn7P-bqRX~X85zSA",
//					"freenet:CHK@7hocoyeLv~6EaaRkpUGWPszeTeUSAwI,UGqEc8F5o6adO9V4wdQ5Ww",
//					"freenet:CHK@zCW-GaT9n60mPXQIehb8VOqbajESAwI,F-sNWoFbfGFF1hG08NujXA",
//					"freenet:CHK@avzkv7GL9w7vJTJjDH7GJIFbrvsSAwI,G4ZgHAny248XesE4jxuTJA",
//					"freenet:CHK@83QeP-2G9Gi2-YH16y6RgbvQqx8SAwI,88xdjXt85tblEVZTEQoqLA"
//			};
//			for (int checkBlockIndex = 0; checkBlockIndex < segmentHeader.getCheckBlockCount(); checkBlockIndex++) {
//				byte[] checkBlockData = encodeSegment.getCheckBlock(checkBlockIndex);
//				FcpGenerateCHK computeCHK = new FcpGenerateCHK(checkBlockData);
//				System.out.println("Computing CHK for Check Block " + checkBlockIndex + "...");
//				fcpClient.execute(computeCHK);
//				assertEquals(checkBlockURIs[checkBlockIndex], computeCHK.getURI());
//				blockMap.setCheckBlock(checkBlockIndex, computeCHK.getURI());
//			}
			
			for (int dataBlockIndex = 0; dataBlockIndex < segmentHeader.getBlockCount(); dataBlockIndex++) {
				byte[] dataBlock = new byte[segmentHeader.getBlockSize()];
				System.arraycopy(data, dataBlockIndex * segmentHeader.getBlockSize(), dataBlock, 0, Math.min(segmentHeader.getBlockSize(), data.length - dataBlockIndex * segmentHeader.getBlockSize()));
				FcpGenerateCHK computeCHK = new FcpGenerateCHK(dataBlock);
				System.out.println("Computing CHK for Data Block " + dataBlockIndex + "...");
				fcpClient.execute(computeCHK);
				blockMap.setDataBlock(dataBlockIndex, computeCHK.getURI());
			}
			
			FcpFecMakeMetadata makeMetadata = new FcpFecMakeMetadata(null, null, new SegmentHeader[] {segmentHeader}, new BlockMap[] {blockMap});
			fcpClient.execute(makeMetadata);
			byte[] metadata = makeMetadata.getMetadata();
			System.out.write(metadata);
			
		} catch (FcpException fcpe1) {
			System.out.println(fcpe1.getFcpMessage().getName());
			throw fcpe1;
		}
	}
	
	public void testFecDecode() throws FcpException, IOException {
		FcpClientGet clientGet = new FcpClientGet("CHK@SMoBZlSea5eXD~igwZROOpp9OscOAwI,AAN2avJs~m61bCPShdF1NQ", 0);
		fcpClient.execute(clientGet);
		byte[] metadata = clientGet.getMetadata();
		FcpFecSegmentSplitFile fecSegmentSplitFile = new FcpFecSegmentSplitFile(metadata);
		fcpClient.execute(fecSegmentSplitFile);
		assertEquals(1, fecSegmentSplitFile.getSegmentHeaderCount());
		assertEquals(fecSegmentSplitFile.getSegmentHeaderCount(), fecSegmentSplitFile.getBlockMapCount());
		for (int segment = 0; segment < fecSegmentSplitFile.getSegmentHeaderCount(); segment++) {
			int segmentOffset = segment * 192;
			SegmentHeader segmentHeader = fecSegmentSplitFile.getSegmentHeader(segment);
			BlockMap blockMap = fecSegmentSplitFile.getBlockMap(segment);
			int blocksRequired = segmentHeader.getBlocksRequired();
			int blocksReceived = 0;
			int dataBlocks = segmentHeader.getBlockCount();
			int checkBlocks = segmentHeader.getCheckBlockCount();
			List receivedDataBlocks = new ArrayList();
			List receivedCheckBlocks = new ArrayList();
			while (blocksReceived < blocksRequired) {
				int blockIndex = (int) (Math.random() * (dataBlocks + checkBlocks));
				String filename = "/tmp/freenet-fcp." + (segmentOffset + blockIndex) + ".data";
				if (new File(filename).exists()) {
					continue;
				}
				String key;
				if (blockIndex < dataBlocks) {
					key = blockMap.getDataBlock(blockIndex);
				} else {
					key = blockMap.getCheckBlock(blockIndex - dataBlocks);
				}
				clientGet = new FcpClientGet(key, 0);
				try {
					fcpClient.execute(clientGet);
				} catch (FcpRuntimeException fre1) {
					continue;
				}
				FileOutputStream fileOutput = new FileOutputStream(filename);
				fileOutput.write(clientGet.getData());
				fileOutput.close();
				blocksReceived++;
				if (blockIndex < dataBlocks) {
					receivedDataBlocks.add(new Integer(blockIndex));
				} else {
					receivedCheckBlocks.add(new Integer(blockIndex));
				}
				System.out.println("Got Block " + blockIndex + " (" + blocksReceived + "/" + blocksRequired + ")");
			}
			Collections.sort(receivedDataBlocks);
			Collections.sort(receivedCheckBlocks);
			List dataInputStreams = new ArrayList();
			List checkInputStreams = new ArrayList();
			for (int dataIndex = 0, dataCount = receivedDataBlocks.size(); dataIndex < dataCount; dataIndex++) {
				String filename = "/tmp/freenet-fcp." + (segmentOffset + ((Integer) receivedDataBlocks.get(dataIndex)).intValue()) + ".data";
				InputStream dataInput = new FileInputStream(filename);
				dataInputStreams.add(dataInput);
			}
			for (int checkIndex = 0, checkCount = receivedCheckBlocks.size(); checkIndex < checkCount; checkIndex++) {
				String filename = "/tmp/freenet-fcp." + (segmentOffset + ((Integer) receivedCheckBlocks.get(checkIndex)).intValue()) + ".data";
				InputStream checkInput = new FileInputStream(filename);
				checkInputStreams.add(checkInput);
			}
			FcpFecDecodeSegment fecDecodeSegment = new FcpFecDecodeSegment(segmentHeader, receivedDataBlocks, receivedCheckBlocks, new SequenceInputStream(Collections.enumeration(dataInputStreams)), new SequenceInputStream(Collections.enumeration(checkInputStreams)));
			fcpClient.execute(fecDecodeSegment);
			int missingBlockCount = fecDecodeSegment.getBlockCount();
			int missingBlockSize = fecDecodeSegment.getBlockSize();
			assertEquals(90 - receivedDataBlocks.size(), missingBlockCount);
			assertEquals(segmentHeader.getBlockSize(), missingBlockSize);
			InputStream missingBlockInput = fecDecodeSegment.getDecodedData();
			for (int missingBlockIndex = 0; missingBlockIndex < missingBlockCount; missingBlockIndex++) {
				int index = fecDecodeSegment.getRequestedBlock(missingBlockIndex);
				String filename = "/tmp/freenet-fcp." + (segmentOffset + index) + ".data";
				OutputStream fileOutput = new FileOutputStream(filename);
				int remaining = fecDecodeSegment.getBlockSize();
				while (remaining > 0) {
					byte[] buffer = new byte[128 * 1024];
					int read = missingBlockInput.read(buffer, 0, Math.min(128 * 1024, remaining));
					if (read != -1) {
						remaining -= read;
						fileOutput.write(buffer, 0, read);
					}
				}
				fileOutput.close();
			}
		}
	}

}
